@@ -2,7 +2,7 @@
 // detail/winrt_utils.hpp
 // ~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2018 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -23,13 +23,12 @@
 #include <cstdlib>
 #include <future>
 #include <locale>
-#include <memory>
 #include <robuffer.h>
 #include <windows.storage.streams.h>
 #include <wrl/implements.h>
 #include <boost/asio/buffer.hpp>
 #include <boost/system/error_code.hpp>
-#include <boost/asio/detail/addressof.hpp>
+#include <boost/asio/detail/memory.hpp>
 #include <boost/asio/detail/socket_ops.hpp>
 
 #include <boost/asio/detail/push_options.hpp>
@@ -84,7 +83,8 @@ inline Windows::Storage::Streams::IBuffer^ buffer_dup(
     const ConstBufferSequence& buffers)
 {
   using Microsoft::WRL::ComPtr;
-  std::size_t size = boost::asio::buffer_size(buffers);
+  using boost::asio::buffer_size;
+  std::size_t size = buffer_size(buffers);
   auto b = ref new Windows::Storage::Streams::Buffer(size);
   ComPtr<IInspectable> insp = reinterpret_cast<IInspectable*>(b);
   ComPtr<Windows::Storage::Streams::IBufferByteAccess> bacc;
