@@ -1,5 +1,6 @@
 /*=============================================================================
     Copyright (c) 2001-2013 Joel de Guzman
+    Copyright (c) 2018 Kohei Takahashi
 
     Distributed under the Boost Software License, Version 1.0. (See accompanying
     file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -7,10 +8,8 @@
 #if !defined(BOOST_FUSION_MAP_DETAIL_VALUE_AT_KEY_IMPL_02042013_0821)
 #define BOOST_FUSION_MAP_DETAIL_VALUE_AT_KEY_IMPL_02042013_0821
 
-#include <boost/fusion/support/detail/access.hpp>
-#include <boost/type_traits/is_const.hpp>
-#include <boost/mpl/at.hpp>
-#include <boost/mpl/identity.hpp>
+#include <boost/fusion/support/config.hpp>
+#include <boost/utility/declval.hpp>
 
 namespace boost { namespace fusion
 {
@@ -27,9 +26,9 @@ namespace boost { namespace fusion
             template <typename Sequence, typename Key>
             struct apply
             {
-                typedef
-                    decltype(std::declval<Sequence>().get_val(mpl::identity<Key>()))
-                type;
+                typedef typename BOOST_FUSION_IDENTIFIED_TYPE((
+                    boost::declval<Sequence>().get_val(mpl::identity<Key>())
+                )) type;
             };
         };
     }
