@@ -5,8 +5,8 @@
 // Copyright (c) 2009-2015 Mateusz Loskot, London, UK.
 // Copyright (c) 2014-2015 Samuel Debionne, Grenoble, France.
 
-// This file was modified by Oracle on 2015, 2016, 2017.
-// Modifications copyright (c) 2015-2017, Oracle and/or its affiliates.
+// This file was modified by Oracle on 2015-2018.
+// Modifications copyright (c) 2015-2018, Oracle and/or its affiliates.
 
 // Contributed and/or modified by Vissarion Fysikopoulos, on behalf of Oracle
 // Contributed and/or modified by Menelaos Karavelas, on behalf of Oracle
@@ -35,6 +35,7 @@
 #include <boost/geometry/core/coordinate_type.hpp>
 #include <boost/geometry/core/tags.hpp>
 
+#include <boost/geometry/util/is_inverse_spheroidal_coordinates.hpp>
 #include <boost/geometry/util/math.hpp>
 #include <boost/geometry/util/select_coordinate_type.hpp>
 
@@ -43,7 +44,6 @@
 
 #include <boost/geometry/algorithms/dispatch/expand.hpp>
 
-
 namespace boost { namespace geometry
 {
 
@@ -112,95 +112,106 @@ struct point_loop_on_spheroid
 
         // normalize input point and input box
         Point p_normalized = detail::return_normalized<Point>(point);
-        detail::normalize(box, box);
 
         // transform input point to be of the same type as the box point
         box_point_type box_point;
         detail::envelope::transform_units(p_normalized, box_point);
 
-        box_coordinate_type p_lon = geometry::get<0>(box_point);
-        box_coordinate_type p_lat = geometry::get<1>(box_point);
-
-        typename coordinate_type<Box>::type
-            b_lon_min = geometry::get<min_corner, 0>(box),
-            b_lat_min = geometry::get<min_corner, 1>(box),
-            b_lon_max = geometry::get<max_corner, 0>(box),
-            b_lat_max = geometry::get<max_corner, 1>(box);
-
-        if (math::is_latitude_pole<units_type, IsEquatorial>(p_lat))
+        if (is_inverse_spheroidal_coordinates(box))
         {
-            // the point of expansion is the either the north or the
-            // south pole; the only important coordinate here is the
-            // pole's latitude, as the longitude can be anything;
-            // we, thus, take into account the point's latitude only and return
-            geometry::set<min_corner, 1>(box, (std::min)(p_lat, b_lat_min));
-            geometry::set<max_corner, 1>(box, (std::max)(p_lat, b_lat_max));
-            return;
-        }
+            geometry::set_from_radian<min_corner, 0>(box, geometry::get_as_radian<0>(p_normalized));
+            geometry::set_from_radian<min_corner, 1>(box, geometry::get_as_radian<1>(p_normalized));
+            geometry::set_from_radian<max_corner, 0>(box, geometry::get_as_radian<0>(p_normalized));
+            geometry::set_from_radian<max_corner, 1>(box, geometry::get_as_radian<1>(p_normalized));
 
-        if (math::equals(b_lat_min, b_lat_max)
-            && math::is_latitude_pole<units_type, IsEquatorial>(b_lat_min))
-        {
-            // the box degenerates to either the north or the south pole;
-            // the only important coordinate here is the pole's latitude, 
-            // as the longitude can be anything;
-            // we thus take into account the box's latitude only and return
-            geometry::set<min_corner, 0>(box, p_lon);
-            geometry::set<min_corner, 1>(box, (std::min)(p_lat, b_lat_min));
-            geometry::set<max_corner, 0>(box, p_lon);
-            geometry::set<max_corner, 1>(box, (std::max)(p_lat, b_lat_max));
-            return;
-        }
+        } else {
 
-        // update latitudes
-        b_lat_min = (std::min)(b_lat_min, p_lat);
-        b_lat_max = (std::max)(b_lat_max, p_lat);
+            detail::normalize(box, box);
 
-        // update longitudes
-        if (math::smaller(p_lon, b_lon_min))
-        {
-            box_coordinate_type p_lon_shifted = p_lon + constants::period();
+            box_coordinate_type p_lon = geometry::get<0>(box_point);
+            box_coordinate_type p_lat = geometry::get<1>(box_point);
 
-            if (math::larger(p_lon_shifted, b_lon_max))
+            typename coordinate_type<Box>::type
+                    b_lon_min = geometry::get<min_corner, 0>(box),
+                    b_lat_min = geometry::get<min_corner, 1>(box),
+                    b_lon_max = geometry::get<max_corner, 0>(box),
+                    b_lat_max = geometry::get<max_corner, 1>(box);
+
+            if (math::is_latitude_pole<units_type, IsEquatorial>(p_lat))
             {
-                // here we could check using: ! math::larger(.., ..)
-                if (math::smaller(b_lon_min - p_lon, p_lon_shifted - b_lon_max))
+                // the point of expansion is the either the north or the
+                // south pole; the only important coordinate here is the
+                // pole's latitude, as the longitude can be anything;
+                // we, thus, take into account the point's latitude only and return
+                geometry::set<min_corner, 1>(box, (std::min)(p_lat, b_lat_min));
+                geometry::set<max_corner, 1>(box, (std::max)(p_lat, b_lat_max));
+                return;
+            }
+
+            if (math::equals(b_lat_min, b_lat_max)
+                    && math::is_latitude_pole<units_type, IsEquatorial>(b_lat_min))
+            {
+                // the box degenerates to either the north or the south pole;
+                // the only important coordinate here is the pole's latitude,
+                // as the longitude can be anything;
+                // we thus take into account the box's latitude only and return
+                geometry::set<min_corner, 0>(box, p_lon);
+                geometry::set<min_corner, 1>(box, (std::min)(p_lat, b_lat_min));
+                geometry::set<max_corner, 0>(box, p_lon);
+                geometry::set<max_corner, 1>(box, (std::max)(p_lat, b_lat_max));
+                return;
+            }
+
+            // update latitudes
+            b_lat_min = (std::min)(b_lat_min, p_lat);
+            b_lat_max = (std::max)(b_lat_max, p_lat);
+
+            // update longitudes
+            if (math::smaller(p_lon, b_lon_min))
+            {
+                box_coordinate_type p_lon_shifted = p_lon + constants::period();
+
+                if (math::larger(p_lon_shifted, b_lon_max))
+                {
+                    // here we could check using: ! math::larger(.., ..)
+                    if (math::smaller(b_lon_min - p_lon, p_lon_shifted - b_lon_max))
+                    {
+                        b_lon_min = p_lon;
+                    }
+                    else
+                    {
+                        b_lon_max = p_lon_shifted;
+                    }
+                }
+            }
+            else if (math::larger(p_lon, b_lon_max))
+            {
+                // in this case, and since p_lon is normalized in the range
+                // (-180, 180], we must have that b_lon_max <= 180
+                if (b_lon_min < 0
+                        && math::larger(p_lon - b_lon_max,
+                                        constants::period() - p_lon + b_lon_min))
                 {
                     b_lon_min = p_lon;
+                    b_lon_max += constants::period();
                 }
                 else
                 {
-                    b_lon_max = p_lon_shifted;
+                    b_lon_max = p_lon;
                 }
             }
-        }
-        else if (math::larger(p_lon, b_lon_max))
-        {
-            // in this case, and since p_lon is normalized in the range
-            // (-180, 180], we must have that b_lon_max <= 180
-            if (b_lon_min < 0
-                && math::larger(p_lon - b_lon_max,
-                                constants::period() - p_lon + b_lon_min))
-            {
-                b_lon_min = p_lon;
-                b_lon_max += constants::period();
-            }
-            else
-            {
-                b_lon_max = p_lon;
-            }
-        }
 
-        geometry::set<min_corner, 0>(box, b_lon_min);
-        geometry::set<min_corner, 1>(box, b_lat_min);
-        geometry::set<max_corner, 0>(box, b_lon_max);
-        geometry::set<max_corner, 1>(box, b_lat_max);
+            geometry::set<min_corner, 0>(box, b_lon_min);
+            geometry::set<min_corner, 1>(box, b_lat_min);
+            geometry::set<max_corner, 0>(box, b_lon_max);
+            geometry::set<max_corner, 1>(box, b_lat_max);
+        }
 
         point_loop
             <
                 2, DimensionCount
             >::apply(box, point, strategy);
-    }
+        }
 };
 
 
