@@ -22,7 +22,7 @@
 #include <boost/mp11/detail/config.hpp>
 #include <boost/mp11/integer_sequence.hpp>
 #include <boost/config.hpp>
-#include <boost/detail/workaround.hpp>
+#include <boost/config/workaround.hpp>
 #include <type_traits>
 #include <utility>
 
@@ -95,6 +95,15 @@ struct list_size_mismatch
 {
 };
 
+#if BOOST_WORKAROUND( BOOST_CUDA_VERSION, >= 9000000 && BOOST_CUDA_VERSION < 10000000 )
+
+template<template<class...> class F, class... L> struct mp_transform_cuda_workaround
+{
+    using type = mp_if<mp_same<mp_size<L>...>, detail::mp_transform_impl<F, L...>, detail::list_size_mismatch>;
+};
+
+#endif
+
 } // namespace detail
 
 #if BOOST_WORKAROUND( BOOST_MSVC, == 1900 ) || BOOST_WORKAROUND( BOOST_GCC, < 40800 )
@@ -103,10 +112,18 @@ template<template<class...> class F, class... L> using mp_transform = typename m
 
 #else
 
+#if BOOST_WORKAROUND( BOOST_CUDA_VERSION, >= 9000000 && BOOST_CUDA_VERSION < 10000000 )
+
+template<template<class...> class F, class... L> using mp_transform = typename detail::mp_transform_cuda_workaround< F, L...>::type::type;
+
+#else
+
 template<template<class...> class F, class... L> using mp_transform = typename mp_if<mp_same<mp_size<L>...>, detail::mp_transform_impl<F, L...>, detail::list_size_mismatch>::type;
 
 #endif
 
+#endif
+
 template<class Q, class... L> using mp_transform_q = mp_transform<Q::template fn, L...>;
 
 namespace detail
@@ -297,15 +314,33 @@ template<template<class...> class L, class... T, std::size_t I> struct mp_at_c_i
 
 template<class L, std::size_t I> struct mp_at_c_impl
 {
-    using _map = mp_transform<mp_list, mp_iota<mp_size<L>>, L>;
-    using type = mp_second<mp_map_find<_map, mp_size_t<I>>>;
+    using _map = mp_transform<mp_list, mp_iota<mp_size<L> >, L>;
+    using type = mp_second<mp_map_find<_map, mp_size_t<I> > >;
+};
+
+#endif
+
+#if BOOST_WORKAROUND( BOOST_CUDA_VERSION, >= 9000000 && BOOST_CUDA_VERSION < 10000000 )
+
+template<class L, std::size_t I> struct mp_at_c_cuda_workaround
+{
+    using type = mp_if_c<(I < mp_size<L>::value), detail::mp_at_c_impl<L, I>, void>;
 };
 
 #endif
 
 } // namespace detail
 
+#if BOOST_WORKAROUND( BOOST_CUDA_VERSION, >= 9000000 && BOOST_CUDA_VERSION < 10000000 )
+
+template<class L, std::size_t I> using mp_at_c = typename detail::mp_at_c_cuda_workaround< L, I >::type::type;
+
+#else
+
 template<class L, std::size_t I> using mp_at_c = typename mp_if_c<(I < mp_size<L>::value), detail::mp_at_c_impl<L, I>, void>::type;
+
+#endif
+
 template<class L, class I> using mp_at = mp_at_c<L, std::size_t{ I::value }>;
 
 // mp_take(_c)<L, N>
@@ -543,6 +578,25 @@ template<template<class...> class L, class T1, class... T, std::size_t I, templa
 
     using L2 = mp_second<part>;
 
+    #if BOOST_WORKAROUND( BOOST_CUDA_VERSION, >= 9000000 && BOOST_CUDA_VERSION < 10000000 )
+
+    struct detail
+    {
+        struct mp_nth_element_impl_cuda_workaround
+        {
+            using type = mp_cond<
+
+                mp_bool<(I < N1)>, mp_nth_element_impl<L1, I, P>,
+                mp_bool<(I == N1)>, mp_identity<T1>,
+                mp_true, mp_nth_element_impl<L2, I - N1 - 1, P>
+
+            >;
+        };
+    };
+    using type = typename detail::mp_nth_element_impl_cuda_workaround::type::type;
+
+    #else
+
     using type = typename mp_cond<
 
         mp_bool<(I < N1)>, mp_nth_element_impl<L1, I, P>,
@@ -550,6 +604,8 @@ template<template<class...> class L, class T1, class... T, std::size_t I, templa
         mp_true, mp_nth_element_impl<L2, I - N1 - 1, P>
 
     >::type;
+
+    #endif
 };
 
 } // namespace detail
@@ -843,7 +899,7 @@ template<template<class...> class L, class T1, class... T, class V, template<cla
 template<template<class...> class L, class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10, class... T, class V, template<class...> class F> struct mp_reverse_fold_impl<L<T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T...>, V, F>
 {
     using rest = typename mp_reverse_fold_impl<L<T...>, V, F>::type;
-    using type = F<T1, F<T2, F<T3, F<T4, F<T5, F<T6, F<T7, F<T8, F<T9, F<T10, rest>>>>>>>>>>;
+    using type = F<T1, F<T2, F<T3, F<T4, F<T5, F<T6, F<T7, F<T8, F<T9, F<T10, rest> > > > > > > > > >;
 };
 
 } // namespace detail
@@ -889,7 +945,7 @@ template<class L, class I, class W> struct mp_replace_at_impl
     template<class T1, class T2> using _p = std::is_same<T2, mp_size_t<I::value>>;
     template<class T1, class T2> using _f = W;
 
-    using type = mp_transform_if<_p, _f, L, mp_iota<mp_size<L>>>;
+    using type = mp_transform_if<_p, _f, L, mp_iota<mp_size<L> > >;
 };
 
 } // namespace detail
@@ -907,15 +963,11 @@ template<class... T, class F> BOOST_CONSTEXPR F mp_for_each_impl( mp_list<T...>,
     return (void)A{ ((void)f(T()), 0)... }, std::forward<F>(f);
 }
 
-#if BOOST_WORKAROUND( BOOST_MSVC, <= 1800 )
-
 template<class F> BOOST_CONSTEXPR F mp_for_each_impl( mp_list<>, F && f )
 {
     return std::forward<F>(f);
 }
 
-#endif
-
 } // namespace detail
 
 template<class L, class F> BOOST_CONSTEXPR F mp_for_each( F && f )
