@@ -1,6 +1,6 @@
 
-// Copyright Christopher Kormanyos 2002 - 2011.
-// Copyright 2011 John Maddock. Distributed under the Boost
+// Copyright Christopher Kormanyos 2002 - 2013.
+// Copyright 2011 - 2013 John Maddock. Distributed under the Boost
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE_1_0.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
@@ -12,6 +12,11 @@
 // This file has no include guards or namespaces - it's expanded inline inside default_ops.hpp
 // 
 
+#ifdef BOOST_MSVC
+#pragma warning(push)
+#pragma warning(disable:6326)  // comparison of two constants
+#endif
+
 namespace detail{
 
 template<typename T, typename U> 
@@ -83,7 +88,7 @@ inline void pow_imp(T& result, const T& t, const U& p, const mpl::true_&)
 } // namespace detail
 
 template<typename T, typename U> 
-inline typename enable_if<is_integral<U> >::type eval_pow(T& result, const T& t, const U& p)
+inline typename enable_if_c<is_integral<U>::value>::type eval_pow(T& result, const T& t, const U& p)
 {
    detail::pow_imp(result, t, p, boost::is_signed<U>());
 }
@@ -98,7 +103,7 @@ void hyp0F0(T& H0F0, const T& x)
    typedef typename mpl::front<typename T::unsigned_types>::type ui_type;
 
    BOOST_ASSERT(&H0F0 != &x);
-   long tol = boost::multiprecision::detail::digits2<number<T, et_on> >::value;
+   long tol = boost::multiprecision::detail::digits2<number<T, et_on> >::value();
    T t;
 
    T x_pow_n_div_n_fact(x);
@@ -112,9 +117,9 @@ void hyp0F0(T& H0F0, const T& x)
 
    ui_type n;
 
-   static const unsigned series_limit = 
-      boost::multiprecision::detail::digits2<number<T, et_on> >::value < 100
-      ? 100 : boost::multiprecision::detail::digits2<number<T, et_on> >::value;
+   const unsigned series_limit = 
+      boost::multiprecision::detail::digits2<number<T, et_on> >::value() < 100
+      ? 100 : boost::multiprecision::detail::digits2<number<T, et_on> >::value();
    // Series expansion of hyperg_0f0(; ; x).
    for(n = 2; n < series_limit; ++n)
    {
@@ -153,16 +158,16 @@ void hyp1F0(T& H1F0, const T& a, const T& x)
    eval_multiply(H1F0, pochham_a, x_pow_n_div_n_fact);
    eval_add(H1F0, si_type(1));
    T lim;
-   eval_ldexp(lim, H1F0, 1 - boost::multiprecision::detail::digits2<number<T, et_on> >::value);
+   eval_ldexp(lim, H1F0, 1 - boost::multiprecision::detail::digits2<number<T, et_on> >::value());
    if(eval_get_sign(lim) < 0)
       lim.negate();
 
    si_type n;
    T term, part;
 
-   static const unsigned series_limit = 
-      boost::multiprecision::detail::digits2<number<T, et_on> >::value < 100
-      ? 100 : boost::multiprecision::detail::digits2<number<T, et_on> >::value;
+   const si_type series_limit =
+      boost::multiprecision::detail::digits2<number<T, et_on> >::value() < 100
+      ? 100 : boost::multiprecision::detail::digits2<number<T, et_on> >::value();
    // Series expansion of hyperg_1f0(a; ; x).
    for(n = 2; n < series_limit; n++)
    {
@@ -200,21 +205,21 @@ void eval_exp(T& result, const T& x)
    // Handle special arguments.
    int type = eval_fpclassify(x);
    bool isneg = eval_get_sign(x) < 0;
-   if(type == FP_NAN)
+   if(type == (int)FP_NAN)
    {
       result = x;
+      errno = EDOM;
       return;
    }
-   else if(type == FP_INFINITE)
+   else if(type == (int)FP_INFINITE)
    {
-      result = x;
       if(isneg)
          result = ui_type(0u);
       else 
          result = x;
       return;
    }
-   else if(type == FP_ZERO)
+   else if(type == (int)FP_ZERO)
    {
       result = ui_type(1);
       return;
@@ -227,23 +232,19 @@ void eval_exp(T& result, const T& x)
       xx.negate();
 
    // Check the range of the argument.
-   static const canonical_exp_type maximum_arg_for_exp = std::numeric_limits<number<T, et_on> >::max_exponent == 0 ? (std::numeric_limits<long>::max)() : std::numeric_limits<number<T, et_on> >::max_exponent;
-
-   if(xx.compare(maximum_arg_for_exp) >= 0)
-   {
-      // Overflow / underflow
-      if(isneg)
-         result = ui_type(0);
-      else
-         result = std::numeric_limits<number<T, et_on> >::has_infinity ? std::numeric_limits<number<T, et_on> >::infinity().backend() : (std::numeric_limits<number<T, et_on> >::max)().backend();
-      return;
-   }
    if(xx.compare(si_type(1)) <= 0)
    {
       //
       // Use series for exp(x) - 1:
       //
-      T lim = std::numeric_limits<number<T, et_on> >::epsilon().backend();
+      T lim;
+      if(std::numeric_limits<number<T, et_on> >::is_specialized)
+         lim = std::numeric_limits<number<T, et_on> >::epsilon().backend();
+      else
+      {
+         result = ui_type(1);
+         eval_ldexp(lim, result, 1 - boost::multiprecision::detail::digits2<number<T, et_on> >::value());
+      }
       unsigned k = 2;
       exp_series = xx;
       result = si_type(1);
@@ -276,6 +277,17 @@ void eval_exp(T& result, const T& x)
       detail::pow_imp(result, get_constant_e<T>(), ll, mpl::true_());
       return;
    }
+   else if(exp_series.compare(x) == 0)
+   {
+      // We have a value that has no fractional part, but is too large to fit 
+      // in a long long, in this situation the code below will fail, so
+      // we're just going to assume that this will overflow:
+      if(isneg)
+         result = ui_type(0);
+      else
+         result = std::numeric_limits<number<T> >::has_infinity ? std::numeric_limits<number<T> >::infinity().backend() : (std::numeric_limits<number<T> >::max)().backend();
+      return;
+   }
 
    // The algorithm for exp has been taken from MPFUN.
    // exp(t) = [ (1 + r + r^2/2! + r^3/3! + r^4/4! ...)^p2 ] * 2^n
@@ -291,8 +303,18 @@ void eval_exp(T& result, const T& x)
    exp_type n;
    eval_convert_to(&n, result);
 
+   if (n == (std::numeric_limits<exp_type>::max)())
+   {
+      // Exponent is too large to fit in our exponent type:
+      if (isneg)
+         result = ui_type(0);
+      else
+         result = std::numeric_limits<number<T> >::has_infinity ? std::numeric_limits<number<T> >::infinity().backend() : (std::numeric_limits<number<T> >::max)().backend();
+      return;
+   }
+
    // The scaling is 2^11 = 2048.
-   static const si_type p2 = static_cast<si_type>(si_type(1) << 11);
+   const si_type p2 = static_cast<si_type>(si_type(1) << 11);
 
    eval_multiply(exp_series, get_constant_ln2<T>(), static_cast<canonical_exp_type>(n));
    eval_subtract(exp_series, xx);
@@ -325,6 +347,29 @@ void eval_log(T& result, const T& arg)
    typedef typename T::exponent_type exp_type;
    typedef typename boost::multiprecision::detail::canonical<exp_type, T>::type canonical_exp_type;
    typedef typename mpl::front<typename T::float_types>::type fp_type;
+   int s = eval_signbit(arg);
+   switch(eval_fpclassify(arg))
+   {
+   case FP_NAN:
+      result = arg;
+      errno = EDOM;
+      return;
+   case FP_INFINITE:
+      if(s) break;
+      result = arg;
+      return;
+   case FP_ZERO:
+      result = std::numeric_limits<number<T> >::has_infinity ? std::numeric_limits<number<T> >::infinity().backend() : (std::numeric_limits<number<T> >::max)().backend();
+      result.negate();
+      errno = ERANGE;
+      return;
+   }
+   if(s)
+   {
+      result = std::numeric_limits<number<T> >::quiet_NaN().backend();
+      errno = EDOM;
+      return;
+   }
 
    exp_type e;
    T t;
@@ -352,7 +397,10 @@ void eval_log(T& result, const T& arg)
    else
       eval_subtract(result, t);
 
-   eval_multiply(lim, result, std::numeric_limits<number<T, et_on> >::epsilon().backend());
+   if(std::numeric_limits<number<T, et_on> >::is_specialized)
+      eval_multiply(lim, result, std::numeric_limits<number<T, et_on> >::epsilon().backend());
+   else
+      eval_ldexp(lim, result, 1 - boost::multiprecision::detail::digits2<number<T, et_on> >::value());
    if(eval_get_sign(lim) < 0)
       lim.negate();
    INSTRUMENT_BACKEND(lim);
@@ -375,14 +423,17 @@ void eval_log(T& result, const T& arg)
 template <class T>
 const T& get_constant_log10()
 {
-   static T result;
-   static bool b = false;
-   if(!b)
+   static BOOST_MP_THREAD_LOCAL T result;
+   static BOOST_MP_THREAD_LOCAL bool b = false;
+   static BOOST_MP_THREAD_LOCAL long digits = boost::multiprecision::detail::digits2<number<T> >::value();
+   if(!b || (digits != boost::multiprecision::detail::digits2<number<T> >::value()))
    {
       typedef typename boost::multiprecision::detail::canonical<unsigned, T>::type ui_type;
       T ten;
       ten = ui_type(10u);
       eval_log(result, ten);
+      b = true;
+      digits = boost::multiprecision::detail::digits2<number<T> >::value();
    }
 
    constant_initializer<T, &get_constant_log10<T> >::do_nothing();
@@ -398,6 +449,13 @@ void eval_log10(T& result, const T& arg)
    eval_divide(result, get_constant_log10<T>());
 }
 
+template <class R, class T>
+inline void eval_log2(R& result, const T& a)
+{
+   eval_log(result, a);
+   eval_divide(result, get_constant_ln2<R>());
+}
+
 template<typename T> 
 inline void eval_pow(T& result, const T& x, const T& a)
 {
@@ -413,19 +471,21 @@ inline void eval_pow(T& result, const T& x, const T& a)
       return;
    }
 
-   if(a.compare(si_type(1)) == 0)
+   if((a.compare(si_type(1)) == 0) || (x.compare(si_type(1)) == 0))
    {
       result = x;
       return;
    }
+   if(a.compare(si_type(0)) == 0)
+   {
+      result = si_type(1);
+      return;
+   }
 
    int type = eval_fpclassify(x);
 
    switch(type)
    {
-   case FP_INFINITE:
-      result = x;
-      return;
    case FP_ZERO:
       switch(eval_fpclassify(a))
       {
@@ -435,13 +495,58 @@ inline void eval_pow(T& result, const T& x, const T& a)
       case FP_NAN:
          result = a;
          break;
+      case FP_NORMAL:
+      {
+         // Need to check for a an odd integer as a special case:
+         try 
+         {
+            typename boost::multiprecision::detail::canonical<boost::intmax_t, T>::type i;
+            eval_convert_to(&i, a);
+            if(a.compare(i) == 0)
+            {
+               if(eval_signbit(a))
+               {
+                  if(i & 1)
+                  {
+                     result = std::numeric_limits<number<T> >::infinity().backend();
+                     if(eval_signbit(x))
+                        result.negate();
+                     errno = ERANGE;
+                  }
+                  else
+                  {
+                     result = std::numeric_limits<number<T> >::infinity().backend();
+                     errno = ERANGE;
+                  }
+               }
+               else if(i & 1)
+               {
+                  result = x;
+               }
+               else
+                  result = si_type(0);
+               return;
+            }
+         }
+         catch(const std::exception&)
+         {
+            // fallthrough..
+         }
+      }
       default:
-         result = x;
+         if(eval_signbit(a))
+         {
+            result = std::numeric_limits<number<T> >::infinity().backend();
+            errno = ERANGE;
+         }
+         else
+            result = x;
          break;
       }
       return;
    case FP_NAN:
       result = x;
+      errno = ERANGE;
       return;
    default: ;
    }
@@ -464,27 +569,42 @@ inline void eval_pow(T& result, const T& x, const T& a)
    }
    
    typename boost::multiprecision::detail::canonical<boost::intmax_t, T>::type an;
+   typename boost::multiprecision::detail::canonical<boost::intmax_t, T>::type max_an =
+      std::numeric_limits<typename boost::multiprecision::detail::canonical<boost::intmax_t, T>::type>::is_specialized ?
+      (std::numeric_limits<typename boost::multiprecision::detail::canonical<boost::intmax_t, T>::type>::max)() :
+      static_cast<typename boost::multiprecision::detail::canonical<boost::intmax_t, T>::type>(1) << (sizeof(typename boost::multiprecision::detail::canonical<boost::intmax_t, T>::type) * CHAR_BIT - 2);
+   typename boost::multiprecision::detail::canonical<boost::intmax_t, T>::type min_an = 
+      std::numeric_limits<typename boost::multiprecision::detail::canonical<boost::intmax_t, T>::type>::is_specialized ?
+      (std::numeric_limits<typename boost::multiprecision::detail::canonical<boost::intmax_t, T>::type>::min)() :
+      -min_an;
+
+
    T fa;
+#ifndef BOOST_NO_EXCEPTIONS
    try
    {
+#endif
       eval_convert_to(&an, a);
       if(a.compare(an) == 0)
       {
          detail::pow_imp(result, x, an, mpl::true_());
          return;
       }
+#ifndef BOOST_NO_EXCEPTIONS
    }
    catch(const std::exception&)
    {
       // conversion failed, just fall through, value is not an integer.
       an = (std::numeric_limits<boost::intmax_t>::max)();
    }
-
+#endif
    if((eval_get_sign(x) < 0))
    {
       typename boost::multiprecision::detail::canonical<boost::uintmax_t, T>::type aun;
+#ifndef BOOST_NO_EXCEPTIONS
       try
       {
+#endif
          eval_convert_to(&aun, a);
          if(a.compare(aun) == 0)
          {
@@ -495,13 +615,40 @@ inline void eval_pow(T& result, const T& x, const T& a)
                result.negate();
             return;
          }
+#ifndef BOOST_NO_EXCEPTIONS
       }
       catch(const std::exception&)
       {
          // conversion failed, just fall through, value is not an integer.
       }
-      if(std::numeric_limits<number<T, et_on> >::has_quiet_NaN)
+#endif
+      eval_floor(result, a);
+      // -1^INF is a special case in C99:
+      if((x.compare(si_type(-1)) == 0) && (eval_fpclassify(a) == FP_INFINITE))
+      {
+         result = si_type(1);
+      }
+      else if(a.compare(result) == 0)
+      {
+         // exponent is so large we have no fractional part:
+         if(x.compare(si_type(-1)) < 0)
+         {
+            result = std::numeric_limits<number<T, et_on> >::infinity().backend();
+         }
+         else
+         {
+            result = si_type(0);
+         }
+      }
+      else if(type == FP_INFINITE)
+      {
+         result = std::numeric_limits<number<T, et_on> >::infinity().backend();
+      }
+      else if(std::numeric_limits<number<T, et_on> >::has_quiet_NaN)
+      {
          result = std::numeric_limits<number<T, et_on> >::quiet_NaN().backend();
+         errno = EDOM;
+      }
       else
       {
          BOOST_THROW_EXCEPTION(std::domain_error("Result of pow is undefined or non-real and there is no NaN for this number type."));
@@ -513,7 +660,7 @@ inline void eval_pow(T& result, const T& x, const T& a)
 
    eval_subtract(da, a, an);
 
-   if((x.compare(fp_type(0.5)) >= 0) && (x.compare(fp_type(0.9)) < 0))
+   if((x.compare(fp_type(0.5)) >= 0) && (x.compare(fp_type(0.9)) < 0) && (an < max_an) && (an > min_an))
    {
       if(a.compare(fp_type(1e-5f)) <= 0)
       {
@@ -568,7 +715,12 @@ inline void eval_pow(T& result, const T& x, const T& a)
 }
 
 template<class T, class A> 
-inline typename enable_if<is_floating_point<A>, void>::type eval_pow(T& result, const T& x, const A& a)
+#if BOOST_WORKAROUND(BOOST_MSVC, < 1800)
+inline typename enable_if_c<!is_integral<A>::value, void>::type 
+#else
+inline typename enable_if_c<is_compatible_arithmetic_type<A, number<T> >::value && !is_integral<A>::value, void>::type 
+#endif
+   eval_pow(T& result, const T& x, const A& a)
 {
    // Note this one is restricted to float arguments since pow.hpp already has a version for
    // integer powers....
@@ -580,7 +732,12 @@ inline typename enable_if<is_floating_point<A>, void>::type eval_pow(T& result,
 }
 
 template<class T, class A> 
-inline typename enable_if<is_arithmetic<A>, void>::type eval_pow(T& result, const A& x, const T& a)
+#if BOOST_WORKAROUND(BOOST_MSVC, < 1800)
+inline void
+#else
+inline typename enable_if_c<is_compatible_arithmetic_type<A, number<T> >::value, void>::type
+#endif
+   eval_pow(T& result, const A& x, const T& a)
 {
    typedef typename boost::multiprecision::detail::canonical<A, T>::type canonical_type;
    typedef typename mpl::if_<is_same<A, canonical_type>, T, canonical_type>::type cast_type;
@@ -589,6 +746,33 @@ inline typename enable_if<is_arithmetic<A>, void>::type eval_pow(T& result, cons
    eval_pow(result, c, a);
 }
 
+template <class T>
+void eval_exp2(T& result, const T& arg)
+{
+   BOOST_STATIC_ASSERT_MSG(number_category<T>::value == number_kind_floating_point, "The log function is only valid for floating point types.");
+
+   // Check for pure-integer arguments which can be either signed or unsigned.
+   typename boost::multiprecision::detail::canonical<typename T::exponent_type, T>::type i;
+   T temp;
+   try {
+      eval_trunc(temp, arg);
+      eval_convert_to(&i, temp);
+      if(arg.compare(i) == 0)
+      {
+         temp = static_cast<typename mpl::front<typename T::unsigned_types>::type>(1u);
+         eval_ldexp(result, temp, i);
+         return;
+      }
+   }
+   catch(const boost::math::rounding_error&)
+   { /* Fallthrough */ }
+   catch(const std::runtime_error&)
+   { /* Fallthrough */ }
+
+   temp = static_cast<typename mpl::front<typename T::unsigned_types>::type>(2u);
+   eval_pow(result, temp, arg);
+}
+
 namespace detail{
 
    template <class T>
@@ -605,7 +789,7 @@ namespace detail{
       ui_type k = 1;
 
       T lim(x);
-      eval_ldexp(lim, lim, 1 - boost::multiprecision::detail::digits2<number<T, et_on> >::value);
+      eval_ldexp(lim, lim, 1 - boost::multiprecision::detail::digits2<number<T, et_on> >::value());
 
       do
       {
@@ -627,6 +811,8 @@ namespace detail{
       switch(eval_fpclassify(x))
       {
       case FP_NAN:
+         errno = EDOM;
+         // fallthrough...
       case FP_INFINITE:
          if(p_sinh)
             *p_sinh = x;
@@ -653,6 +839,8 @@ namespace detail{
          T e_px, e_mx;
          eval_exp(e_px, x);
          eval_divide(e_mx, ui_type(1), e_px);
+         if(eval_signbit(e_mx) != eval_signbit(e_px))
+            e_mx.negate();  // Handles lack of signed zero in some types
 
          if(p_sinh) 
          { 
@@ -700,6 +888,17 @@ inline void eval_tanh(T& result, const T& x)
    BOOST_STATIC_ASSERT_MSG(number_category<T>::value == number_kind_floating_point, "The tanh function is only valid for floating point types.");
   T c;
   detail::sinhcosh(x, &result, &c);
+  if((eval_fpclassify(result) == FP_INFINITE) && (eval_fpclassify(c) == FP_INFINITE))
+  {
+     bool s = eval_signbit(result) != eval_signbit(c);
+     result = static_cast<typename mpl::front<typename T::unsigned_types>::type>(1u);
+     if(s)
+        result.negate();
+     return;
+  }
   eval_divide(result, c);
 }
 
+#ifdef BOOST_MSVC
+#pragma warning(pop)
+#endif
