@@ -2,7 +2,7 @@
 // timer.cpp
 // ~~~~~~~~~
 //
-// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2018 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -11,17 +11,16 @@
 #include <iostream>
 #include <boost/asio.hpp>
 #include <boost/bind.hpp>
-#include <boost/date_time/posix_time/posix_time.hpp>
 
 void print(const boost::system::error_code& /*e*/,
-    boost::asio::deadline_timer* t, int* count)
+    boost::asio::steady_timer* t, int* count)
 {
   if (*count < 5)
   {
     std::cout << *count << std::endl;
     ++(*count);
 
-    t->expires_at(t->expires_at() + boost::posix_time::seconds(1));
+    t->expires_at(t->expiry() + boost::asio::chrono::seconds(1));
     t->async_wait(boost::bind(print,
           boost::asio::placeholders::error, t, count));
   }
@@ -29,10 +28,10 @@ void print(const boost::system::error_code& /*e*/,
 
 int main()
 {
-  boost::asio::io_service io;
+  boost::asio::io_context io;
 
   int count = 0;
-  boost::asio::deadline_timer t(io, boost::posix_time::seconds(1));
+  boost::asio::steady_timer t(io, boost::asio::chrono::seconds(1));
   t.async_wait(boost::bind(print,
         boost::asio::placeholders::error, &t, &count));
 
