@@ -16,12 +16,15 @@
 #include <boost/beast/http/message.hpp>
 #include <boost/beast/http/read.hpp>
 #include <boost/beast/http/string_body.hpp>
+#include <boost/beast/core/buffers_to_string.hpp>
 #include <boost/beast/core/error.hpp>
 #include <boost/beast/core/multi_buffer.hpp>
-#include <boost/beast/test/stream.hpp>
+#include <boost/beast/experimental/test/stream.hpp>
 #include <boost/beast/test/yield_to.hpp>
 #include <boost/beast/unit_test/suite.hpp>
 #include <boost/asio/error.hpp>
+#include <boost/asio/io_service.hpp>
+#include <boost/asio/strand.hpp>
 #include <sstream>
 #include <string>
 
@@ -47,10 +50,10 @@ public:
                 boost::asio::const_buffer;
 
             template<bool isRequest, class Fields>
-            explicit
-            writer(message<isRequest,
-                    unsized_body, Fields> const& msg)
-                : body_(msg.body())
+            writer(
+                header<isRequest, Fields> const&,
+                value_type const& b)
+                : body_(b)
             {
             }
 
@@ -92,10 +95,10 @@ public:
                 boost::asio::const_buffer;
 
             template<bool isRequest, class Fields>
-            explicit
-            writer(message<isRequest,
-                    test_body, Fields> const& msg)
-                : body_(msg.body())
+            writer(
+                header<isRequest, Fields> const&,
+                value_type const& b)
+                : body_(b)
             {
             }
 
@@ -202,11 +205,11 @@ public:
             friend class writer;
 
             std::string s_;
-            test::fail_counter& fc_;
+            test::fail_count& fc_;
 
         public:
             explicit
-            value_type(test::fail_counter& fc)
+            value_type(test::fail_count& fc)
                 : fc_(fc)
             {
             }
@@ -230,9 +233,8 @@ public:
 
             template<bool isRequest, class Fields>
             explicit
-            writer(message<isRequest,
-                    fail_body, Fields> const& msg)
-                : body_(msg.body())
+            writer(header<isRequest, Fields> const&, value_type const& b)
+                : body_(b)
             {
             }
 
@@ -265,19 +267,6 @@ public:
         return ss.str();
     }
 
-    template<class ConstBufferSequence>
-    static
-    std::string
-    to_string(ConstBufferSequence const& bs)
-    {
-        std::string s;
-        s.reserve(buffer_size(bs));
-        for(auto b : beast::detail::buffers_range(bs))
-            s.append(reinterpret_cast<char const*>(b),
-                b.size());
-        return s;
-    }
-
     template<bool isRequest>
     bool
     equal_body(string_view sv, string_view body)
@@ -366,7 +355,7 @@ public:
 
         for(n = 0; n < limit; ++n)
         {
-            test::fail_counter fc(n);
+            test::fail_count fc(n);
             test::stream ts{ioc_, fc}, tr{ioc_};
             ts.connect(tr);
             request<fail_body> m(verb::get, "/", 10, fc);
@@ -396,14 +385,14 @@ public:
 
         for(n = 0; n < limit; ++n)
         {
-            test::fail_counter fc(n);
+            test::fail_count fc(n);
             test::stream ts{ioc_, fc}, tr{ioc_};
             ts.connect(tr);
             request<fail_body> m{verb::get, "/", 10, fc};
             m.set(field::user_agent, "test");
             m.set(field::transfer_encoding, "chunked");
             m.body() = "*****";
-            error_code ec = test::error::fail_error;
+            error_code ec = test::error::test_failure;
             write(ts, m, ec);
             if(! ec)
             {
@@ -427,14 +416,14 @@ public:
 
         for(n = 0; n < limit; ++n)
         {
-            test::fail_counter fc(n);
+            test::fail_count fc(n);
             test::stream ts{ioc_, fc}, tr{ioc_};
             ts.connect(tr);
             request<fail_body> m{verb::get, "/", 10, fc};
             m.set(field::user_agent, "test");
             m.set(field::transfer_encoding, "chunked");
             m.body() = "*****";
-            error_code ec = test::error::fail_error;
+            error_code ec = test::error::test_failure;
             async_write(ts, m, do_yield[ec]);
             if(! ec)
             {
@@ -458,7 +447,7 @@ public:
 
         for(n = 0; n < limit; ++n)
         {
-            test::fail_counter fc(n);
+            test::fail_count fc(n);
             test::stream ts{ioc_, fc}, tr{ioc_};
             ts.connect(tr);
             request<fail_body> m{verb::get, "/", 10, fc};
@@ -466,7 +455,7 @@ public:
             m.set(field::connection, "keep-alive");
             m.set(field::content_length, "5");
             m.body() = "*****";
-            error_code ec = test::error::fail_error;
+            error_code ec = test::error::test_failure;
             write(ts, m, ec);
             if(! ec)
             {
@@ -485,7 +474,7 @@ public:
 
         for(n = 0; n < limit; ++n)
         {
-            test::fail_counter fc(n);
+            test::fail_count fc(n);
             test::stream ts{ioc_, fc}, tr{ioc_};
             ts.connect(tr);
             request<fail_body> m{verb::get, "/", 10, fc};
@@ -493,7 +482,7 @@ public:
             m.set(field::connection, "keep-alive");
             m.set(field::content_length, "5");
             m.body() = "*****";
-            error_code ec = test::error::fail_error;
+            error_code ec = test::error::test_failure;
             async_write(ts, m, do_yield[ec]);
             if(! ec)
             {
@@ -768,7 +757,6 @@ public:
             }
             {
                 auto m = m0;
-                error_code ec;
                 response_serializer<Body, fields> sr{m};
                 sr.split(true);
                 for(;;)
@@ -782,7 +770,6 @@ public:
             }
             {
                 auto m = m0;
-                error_code ec;
                 response_serializer<Body, fields> sr{m};
                 sr.split(true);
                 for(;;)
@@ -815,7 +802,6 @@ public:
             }
             {
                 auto m = m0;
-                error_code ec;
                 response_serializer<Body, fields> sr{m};
                 sr.split(true);
                 for(;;)
@@ -829,7 +815,6 @@ public:
             }
             {
                 auto m = m0;
-                error_code ec;
                 response_serializer<Body, fields> sr{m};
                 sr.split(true);
                 for(;;)
@@ -860,6 +845,150 @@ public:
         ioc.run();
     }
 
+    struct copyable_handler
+    {
+        template<class... Args>
+        void
+        operator()(Args&&...) const
+        {
+        }
+    };
+
+    void
+    testAsioHandlerInvoke()
+    {
+        // make sure things compile, also can set a
+        // breakpoint in asio_handler_invoke to make sure
+        // it is instantiated.
+        {
+            boost::asio::io_context ioc;
+            boost::asio::io_service::strand s{ioc};
+            test::stream ts{ioc};
+            flat_buffer b;
+            request<empty_body> m;
+            request_serializer<empty_body, fields> sr{m};
+            async_write_some(ts, sr, s.wrap(copyable_handler{}));
+        }
+        {
+            boost::asio::io_context ioc;
+            boost::asio::io_service::strand s{ioc};
+            test::stream ts{ioc};
+            flat_buffer b;
+            request<empty_body> m;
+            request_serializer<empty_body, fields> sr{m};
+            async_write(ts, sr, s.wrap(copyable_handler{}));
+        }
+        {
+            boost::asio::io_context ioc;
+            boost::asio::io_service::strand s{ioc};
+            test::stream ts{ioc};
+            flat_buffer b;
+            request<empty_body> m;
+            async_write(ts, m, s.wrap(copyable_handler{}));
+        }
+    }
+
+    struct const_body_writer
+    {
+        struct value_type{};
+
+        struct writer
+        {
+            using const_buffers_type =
+                boost::asio::const_buffer;
+
+            template<bool isRequest, class Fields>
+            writer(
+                header<isRequest, Fields> const&,
+                value_type const&)
+            {
+            }
+
+            void
+            init(error_code& ec)
+            {
+                ec.assign(0, ec.category());
+            }
+
+            boost::optional<std::pair<const_buffers_type, bool>>
+            get(error_code& ec)
+            {
+                ec.assign(0, ec.category());
+                return {{const_buffers_type{"", 0}, false}};
+            }
+        };
+    };
+
+    struct mutable_body_writer
+    {
+        struct value_type{};
+
+        struct writer
+        {
+            using const_buffers_type =
+                boost::asio::const_buffer;
+
+            template<bool isRequest, class Fields>
+            writer(
+                header<isRequest, Fields>&,
+                value_type&)
+            {
+            }
+
+            void
+            init(error_code& ec)
+            {
+                ec.assign(0, ec.category());
+            }
+
+            boost::optional<std::pair<const_buffers_type, bool>>
+            get(error_code& ec)
+            {
+                ec.assign(0, ec.category());
+                return {{const_buffers_type{"", 0}, false}};
+            }
+        };
+    };
+
+    void
+    testBodyWriters()
+    {
+        {
+            test::stream s{ioc_};
+            message<true, const_body_writer> m;
+            try
+            {
+                write(s, m);
+            }
+            catch(std::exception const&)
+            {
+            }
+        }
+        {
+            error_code ec;
+            test::stream s{ioc_};
+            message<true, const_body_writer> m;
+            write(s, m, ec);
+        }
+        {
+            test::stream s{ioc_};
+            message<true, mutable_body_writer> m;
+            try
+            {
+                write(s, m);
+            }
+            catch(std::exception const&)
+            {
+            }
+        }
+        {
+            error_code ec;
+            test::stream s{ioc_};
+            message<true, mutable_body_writer> m;
+            write(s, m, ec);
+        }
+    }
+
     void
     run() override
     {
@@ -881,6 +1010,8 @@ public:
                 testWriteStream<test_body< true, false>>(yield);
                 testWriteStream<test_body< true,  true>>(yield);
             });
+        testAsioHandlerInvoke();
+        testBodyWriters();
     }
 };
 
