@@ -442,7 +442,8 @@ struct test_envelope_on_sphere_or_spheroid<Geometry, MBR, bg::ring_tag, TestReve
             <
                 MBR
             >::apply(case_id, geometry,
-                     lon_min1, lat_min1, lon_max1, lat_max1,
+                     lon_min1, lat_min1, 0,
+                     lon_max1, lat_max1, 0,
                      tolerance);
 
         std::string ccw_case_id = case_id + "-2ccw";
@@ -457,7 +458,8 @@ struct test_envelope_on_sphere_or_spheroid<Geometry, MBR, bg::ring_tag, TestReve
             <
                 MBR
             >::apply(ccw_case_id, ccw_ring,
-                     lon_min2, lat_min2, lon_max2, lat_max2,
+                     lon_min2, lat_min2, 0,
+                     lon_max2, lat_max2, 0,
                      tolerance);
 
 #ifdef BOOST_GEOMETRY_TEST_DEBUG
@@ -805,6 +807,32 @@ BOOST_AUTO_TEST_CASE( envelope_segment_sphere )
                   1-heps, 1, 1, 2);
 }
 
+BOOST_AUTO_TEST_CASE( envelope_segment_spherical_polar )
+{
+    typedef bg::cs::spherical<bg::degree> coordinate_system_type;
+    typedef bg::model::point<double, 2, coordinate_system_type> P;
+    typedef bg::model::segment<P> G;
+    typedef bg::model::box<P> B;
+    typedef test_envelope_on_sphere_or_spheroid<G, B> tester;
+
+    tester::apply("s01",
+                  from_wkt<G>("SEGMENT(10 10,40 40)"),
+                  10, 10, 40, 40);
+
+    tester::apply("s02",
+                  from_wkt<G>("SEGMENT(10 80,40 80)"),
+                  10, 90 - 10.345270046149988, 40, 80);
+
+    tester::apply("s03",
+                  from_wkt<G>("SEGMENT(160 80,-170 80)"),
+                  160, 90 - 10.34527004614999, 190, 80);
+
+    // segment ending at the north pole
+    tester::apply("s05",
+                  from_wkt<G>("SEGMENT(40 45,80 0)"),
+                  40, 0, 40, 45);
+}
+
 
 BOOST_AUTO_TEST_CASE( envelope_segment_spheroid )
 {
@@ -1292,6 +1320,22 @@ BOOST_AUTO_TEST_CASE( envelope_segment_spheroid_with_strategy_andoyer )
                   from_wkt<G>("SEGMENT(100 80, 20 10)"),
                   20, 10, 100, 80);
 
+    // segments intersecting pole
+    tester::apply("s19",
+                  from_wkt<G>("SEGMENT(0 0, 180 0)"),
+                  0, 0, 180, 90);
+    tester::apply("s20",
+                  from_wkt<G>("SEGMENT(0 0, -180 0)"),
+                  0, 0, 180, 90);
+    tester::apply("s21",
+                  from_wkt<G>("SEGMENT(0 1, 180 1)"),
+                  0, 1, 180, 90,
+                  std::numeric_limits<double>::epsilon() * 10);
+    tester::apply("s22",
+                  from_wkt<G>("SEGMENT(0 -1, 180 -1)"),
+                  0, -90, 180, -1,
+                  std::numeric_limits<double>::epsilon() * 10);
+
 }
 
 BOOST_AUTO_TEST_CASE( envelope_segment_spheroid_with_strategy_vincenty )
@@ -1667,8 +1711,7 @@ BOOST_AUTO_TEST_CASE( envelope_multipoint_with_height )
 template <typename CoordinateSystem>
 void test_envelope_box()
 {
-    typedef bg::cs::spherical_equatorial<bg::degree> coordinate_system_type;
-    typedef bg::model::point<double, 2, coordinate_system_type> P;
+    typedef bg::model::point<double, 2, CoordinateSystem> P;
     typedef bg::model::box<P> G;
     typedef bg::model::box<P> B;
     typedef test_envelope_on_sphere_or_spheroid<G, B> tester;
@@ -1857,10 +1900,36 @@ void test_envelope_box()
     tester::apply("b100", G(P(1-heps, 1-heps), P(1, 1)), 1-heps, 1-heps, 1, 1);
 }
 
+template <typename CoordinateSystem>
+void test_envelope_box_polar()
+{
+    typedef bg::model::point<double, 2, CoordinateSystem> P;
+    typedef bg::model::box<P> G;
+    typedef bg::model::box<P> B;
+    typedef test_envelope_on_sphere_or_spheroid<G, B> tester;
+
+    tester::apply("b01",
+                  from_wkt<G>("BOX(10 10,20 20)"),
+                  10, 10, 20, 20);
+
+    tester::apply("b02a",
+                  from_wkt<G>("BOX(170 10,-170 20)"),
+                  170, 10, 190, 20);
+
+    tester::apply("b10b",
+                  from_wkt<G>("BOX(0 0,10 180)"),
+                  0, 0, 10, 180);
+
+    tester::apply("b16a",
+                  from_wkt<G>("BOX(170 40,-170 180)"),
+                  170, 40, 190, 180);
+}
+
 BOOST_AUTO_TEST_CASE( envelope_box )
 {
     test_envelope_box<bg::cs::spherical_equatorial<bg::degree> >();
     test_envelope_box<bg::cs::geographic<bg::degree> >();
+    test_envelope_box_polar<bg::cs::spherical<bg::degree> >();
 }
 
 
@@ -2366,10 +2435,46 @@ BOOST_AUTO_TEST_CASE( envelope_multilinestring_spheroid_with_height )
                   -10, 25, 300, 260, 80, 700);
 }
 
+//Test spherical polygons and rings (geographic should be similar)
+BOOST_AUTO_TEST_CASE( envelope_polygon )
+{
+    typedef bg::cs::spherical_equatorial<bg::degree> coordinate_system_type;
+    typedef bg::model::point<double, 2, coordinate_system_type> point_type;
+    typedef bg::model::polygon<point_type> G;
+    typedef bg::model::box<point_type> B;
+    typedef test_envelope_on_sphere_or_spheroid<G, B> tester;
+
+    typedef bg::model::ring<point_type> R;
+    typedef test_envelope_on_sphere_or_spheroid<R, B> testerR;
+    R ring1;
+    bg::append(ring1, point_type(0.0, 0.0));
+    bg::append(ring1, point_type(0.0, 5.0));
+    bg::append(ring1, point_type(5.0, 5.0));
+    bg::append(ring1, point_type(5.0, 0.0));
+    bg::append(ring1, point_type(0.0, 0.0));
+
+    testerR::apply("r01",
+                  ring1,
+                  0, 0, 5, 5.0047392446083938);
+    tester::apply("p01",
+                  from_wkt<G>("POLYGON((0 0,1 0,1 1,0 1,0 0))"),
+                  0, 0, 1, 1.0000380706527705);
+    tester::apply("p02",
+                  from_wkt<G>("POLYGON((0 0,1 0,1 1,0 1,0 0),(0.5 0.5,0.7 0.5,0.7 0.7,0.5 0.5))"),
+                  0, 0, 1, 1.0000380706527705);
+    tester::apply("p03",
+                  from_wkt<G>("POLYGON((),(0.5 0.5,0.5 0.7,0.7 0.7,0.5 0.5))"),
+                  0.5, 0.5, 0.7, 0.70000106605644807);
+    tester::apply("p04",
+                  from_wkt<G>("POLYGON((),(0.5 0.5,0.5 0.7,0.7 0.7,0.5 0.5),\
+                              (0.7 0.5,0.9 0.5,0.9 0.7,0.7 0.5))"),
+                  0.5, 0.5, 0.9, 0.70000106605644807);
+}
 
 // unit test for rings de-activated for now (current implementation
 // for area on the spherical equatorial coordinate system is not complete)
 // TODO: re-activate once implementation is done
+// right now implementation does not distinguish between ccw and cw rings
 BOOST_AUTO_TEST_CASE( envelope_cw_ring )
 {
     typedef bg::cs::spherical_equatorial<bg::degree> coordinate_system_type;
