@@ -2,6 +2,7 @@
 // Unit Test
 
 // Copyright (c) 2007-2015 Barend Gehrels, Amsterdam, the Netherlands.
+// Copyright (c) 2017 Adam Wulkiewicz, Lodz, Poland.
 
 // This file was modified by Oracle on 2016, 2017.
 // Modifications copyright (c) 2016-2017, Oracle and/or its affiliates.
@@ -104,8 +105,8 @@ void difference_output(std::string const& caseid, G1 const& g1, G2 const& g2, Ou
             << string_from_type<coordinate_type>::name()
             << (ccw ? "_ccw" : "")
             << (open ? "_open" : "")
-#if defined(BOOST_GEOMETRY_INCLUDE_SELF_TURNS)
-           << "_self"
+#if defined(BOOST_GEOMETRY_NO_SELF_TURNS)
+           << "_no_self"
 #endif
 #if defined(BOOST_GEOMETRY_NO_ROBUSTNESS)
             << "_no_rob"
@@ -253,7 +254,7 @@ std::string test_difference(std::string const& caseid, G1 const& g1, G2 const& g
 
     if (expected_count >= 0)
     {
-        BOOST_CHECK_MESSAGE(int(result.size()) == expected_count,
+        BOOST_CHECK_MESSAGE(int(boost::size(result)) == expected_count,
                 "difference: " << caseid
                 << " #outputs expected: " << expected_count
                 << " detected: " << result.size()
@@ -263,7 +264,7 @@ std::string test_difference(std::string const& caseid, G1 const& g1, G2 const& g
 
     if (expected_rings_count >= 0)
     {
-        int nrings = expected_count + bg::num_interior_rings(result);
+        int nrings = int(boost::size(result) + bg::num_interior_rings(result));
         BOOST_CHECK_MESSAGE(nrings == expected_rings_count,
                 "difference: " << caseid
                 << " #rings expected: " << expected_rings_count
@@ -426,6 +427,7 @@ void test_one_lp(std::string const& caseid,
     bg::read_wkt(wkt2, g2);
 
     bg::correct(g1);
+    bg::correct(g2);
 
     typedef typename setop_output_type<OutputType>::type result_type;
     result_type pieces;
