@@ -45,6 +45,9 @@
 #include <boost/gil/typedefs.hpp>
 #include <boost/gil/algorithm.hpp>
 
+#include <boost/test/unit_test.hpp>
+
+
 using namespace boost::gil;
 
 // returns time in milliseconds per call
@@ -81,9 +84,9 @@ struct fill_nongil_t<RGB_VIEW(T), P> {
         T* first=(T*)_v.row_begin(0);
         T* last=first+_v.size()*3;
         while(first!=last) {
-            first[0]=at_c<0>(_p);
-            first[1]=at_c<1>(_p);
-            first[2]=at_c<2>(_p);
+            first[0]=boost::gil::at_c<0>(_p);
+            first[1]=boost::gil::at_c<1>(_p);
+            first[2]=boost::gil::at_c<2>(_p);
             first+=3;
         }
     }
@@ -99,9 +102,9 @@ struct fill_nongil_t<RGB_VIEW(T1), pixel<T2,bgr_layout_t> > {
         T1* first=(T1*)_v.row_begin(0);
         T1* last=first+_v.size()*3;
         while(first!=last) {
-            first[0]=at_c<2>(_p);
-            first[1]=at_c<1>(_p);
-            first[2]=at_c<0>(_p);
+            first[0]=boost::gil::at_c<2>(_p);
+            first[1]=boost::gil::at_c<1>(_p);
+            first[2]=boost::gil::at_c<0>(_p);
             first+=3;
         }
     }
@@ -116,12 +119,12 @@ struct fill_nongil_t<RGB_PLANAR_VIEW(T1), pixel<T2,rgb_layout_t> > {
     void operator()() const {
         std::size_t size=_v.size();
         T1* first;
-        first=(T1*)at_c<0>(_v.row_begin(0));
-        std::fill(first,first+size,at_c<0>(_p));
-        first=(T1*)at_c<1>(_v.row_begin(0));
-        std::fill(first,first+size,at_c<1>(_p));
-        first=(T1*)at_c<2>(_v.row_begin(0));
-        std::fill(first,first+size,at_c<2>(_p));
+        first=(T1*)boost::gil::at_c<0>(_v.row_begin(0));
+        std::fill(first,first+size,boost::gil::at_c<0>(_p));
+        first=(T1*)boost::gil::at_c<1>(_v.row_begin(0));
+        std::fill(first,first+size,boost::gil::at_c<1>(_p));
+        first=(T1*)boost::gil::at_c<2>(_v.row_begin(0));
+        std::fill(first,first+size,boost::gil::at_c<2>(_p));
     }
 };
 
@@ -135,12 +138,12 @@ struct fill_nongil_t<RGB_PLANAR_VIEW(T1), pixel<T2,bgr_layout_t> > {
     void operator()() const {
         std::size_t size=_v.size();
         T1* first;
-        first=(T1*)at_c<0>(_v.row_begin(0));
-        std::fill(first,first+size,at_c<2>(_p));
-        first=(T1*)at_c<1>(_v.row_begin(0));
-        std::fill(first,first+size,at_c<1>(_p));
-        first=(T1*)at_c<2>(_v.row_begin(0));
-        std::fill(first,first+size,at_c<1>(_p));
+        first=(T1*)boost::gil::at_c<0>(_v.row_begin(0));
+        std::fill(first,first+size,boost::gil::at_c<2>(_p));
+        first=(T1*)boost::gil::at_c<1>(_v.row_begin(0));
+        std::fill(first,first+size,boost::gil::at_c<1>(_p));
+        first=(T1*)boost::gil::at_c<2>(_v.row_begin(0));
+        std::fill(first,first+size,boost::gil::at_c<1>(_p));
     }
 };
 
@@ -191,9 +194,9 @@ struct for_each_nongil_t<RGB_PLANAR_VIEW(T1), rgb_fr_t<T2> > {
     for_each_nongil_t(const View& v_in,const F& f_in) : _v(v_in), _f(f_in) {}
     void operator()() const {
         T1 *first0, *first1, *first2, *last0;
-        first0=(T1*)at_c<0>(_v.row_begin(0));
-        first1=(T1*)at_c<1>(_v.row_begin(0));
-        first2=(T1*)at_c<2>(_v.row_begin(0));
+        first0=(T1*)boost::gil::at_c<0>(_v.row_begin(0));
+        first1=(T1*)boost::gil::at_c<1>(_v.row_begin(0));
+        first2=(T1*)boost::gil::at_c<2>(_v.row_begin(0));
         last0=first0+_v.size();
         while(first0!=last0) {
             *first0++=0;
@@ -261,12 +264,12 @@ struct copy_nongil_t<RGB_PLANAR_VIEW(T1),RGB_PLANAR_VIEW(T2)> {
     copy_nongil_t(const View1& v1_in,const View2& v2_in) : _v1(v1_in), _v2(v2_in) {}
     void operator()() const {
         std::size_t size=_v1.size();
-        T1* first10=(T1*)at_c<0>(_v1.row_begin(0));
-        T1* first11=(T1*)at_c<1>(_v1.row_begin(0));
-        T1* first12=(T1*)at_c<2>(_v1.row_begin(0));
-        T2* first20=(T2*)at_c<0>(_v2.row_begin(0));
-        T2* first21=(T2*)at_c<1>(_v2.row_begin(0));
-        T2* first22=(T2*)at_c<2>(_v2.row_begin(0));
+        T1* first10=(T1*)boost::gil::at_c<0>(_v1.row_begin(0));
+        T1* first11=(T1*)boost::gil::at_c<1>(_v1.row_begin(0));
+        T1* first12=(T1*)boost::gil::at_c<2>(_v1.row_begin(0));
+        T2* first20=(T2*)boost::gil::at_c<0>(_v2.row_begin(0));
+        T2* first21=(T2*)boost::gil::at_c<1>(_v2.row_begin(0));
+        T2* first22=(T2*)boost::gil::at_c<2>(_v2.row_begin(0));
         std::copy(first10,first10+size,first20);
         std::copy(first11,first11+size,first21);
         std::copy(first12,first12+size,first22);
@@ -282,9 +285,9 @@ struct copy_nongil_t<RGB_VIEW(T1),RGB_PLANAR_VIEW(T2)> {
     void operator()() const {
         T1* first=(T1*)_v1.row_begin(0);
         T1* last=first+_v1.size()*3;
-        T2* first0=(T2*)at_c<0>(_v2.row_begin(0));
-        T2* first1=(T2*)at_c<1>(_v2.row_begin(0));
-        T2* first2=(T2*)at_c<2>(_v2.row_begin(0));
+        T2* first0=(T2*)boost::gil::at_c<0>(_v2.row_begin(0));
+        T2* first1=(T2*)boost::gil::at_c<1>(_v2.row_begin(0));
+        T2* first2=(T2*)boost::gil::at_c<2>(_v2.row_begin(0));
         while(first!=last) {
             *first0++=first[0];
             *first1++=first[1];
@@ -303,9 +306,9 @@ struct copy_nongil_t<RGB_PLANAR_VIEW(T1),RGB_VIEW(T2)> {
     void operator()() const {
         T1* first=(T1*)_v2.row_begin(0);
         T1* last=first+_v2.size()*3;
-        T2* first0=(T2*)at_c<0>(_v1.row_begin(0));
-        T2* first1=(T2*)at_c<1>(_v1.row_begin(0));
-        T2* first2=(T2*)at_c<2>(_v1.row_begin(0));
+        T2* first0=(T2*)boost::gil::at_c<0>(_v1.row_begin(0));
+        T2* first1=(T2*)boost::gil::at_c<1>(_v1.row_begin(0));
+        T2* first2=(T2*)boost::gil::at_c<2>(_v1.row_begin(0));
         while(first!=last) {
             first[0]=*first0++;
             first[1]=*first1++;
@@ -369,12 +372,12 @@ struct transform_nongil_t<RGB_PLANAR_VIEW(T1),RGB_PLANAR_VIEW(T2),F> {
     F _f;
     transform_nongil_t(const View1& v1_in,const View2& v2_in,const F& f_in) : _v1(v1_in),_v2(v2_in),_f(f_in) {}
     void operator()() const {
-        T1* first10=(T1*)at_c<0>(_v1.row_begin(0));
-        T1* first11=(T1*)at_c<1>(_v1.row_begin(0));
-        T1* first12=(T1*)at_c<2>(_v1.row_begin(0));
-        T1* first20=(T2*)at_c<0>(_v2.row_begin(0));
-        T1* first21=(T2*)at_c<1>(_v2.row_begin(0));
-        T1* first22=(T2*)at_c<2>(_v2.row_begin(0));
+        T1* first10=(T1*)boost::gil::at_c<0>(_v1.row_begin(0));
+        T1* first11=(T1*)boost::gil::at_c<1>(_v1.row_begin(0));
+        T1* first12=(T1*)boost::gil::at_c<2>(_v1.row_begin(0));
+        T1* first20=(T2*)boost::gil::at_c<0>(_v2.row_begin(0));
+        T1* first21=(T2*)boost::gil::at_c<1>(_v2.row_begin(0));
+        T1* first22=(T2*)boost::gil::at_c<2>(_v2.row_begin(0));
         T1* last10=first10+_v1.size();
         while(first10!=last10) {
             *first20++=T2(*first12++*0.1f);
@@ -394,9 +397,9 @@ struct transform_nongil_t<RGB_VIEW(T1),RGB_PLANAR_VIEW(T2),F> {
     void operator()() const {
         T1* first1=(T1*)_v1.row_begin(0);
         T1* last1=first1+_v1.size()*3;
-        T1* first20=(T2*)at_c<0>(_v2.row_begin(0));
-        T1* first21=(T2*)at_c<1>(_v2.row_begin(0));
-        T1* first22=(T2*)at_c<2>(_v2.row_begin(0));
+        T1* first20=(T2*)boost::gil::at_c<0>(_v2.row_begin(0));
+        T1* first21=(T2*)boost::gil::at_c<1>(_v2.row_begin(0));
+        T1* first22=(T2*)boost::gil::at_c<2>(_v2.row_begin(0));
         while(first1!=last1) {
             *first20++=T2(first1[2]*0.1f);
             *first21++=T2(first1[1]*0.2f);
@@ -414,9 +417,9 @@ struct transform_nongil_t<RGB_PLANAR_VIEW(T1),RGB_VIEW(T2),F> {
     F _f;
     transform_nongil_t(const View1& v1_in,const View2& v2_in,const F& f_in) : _v1(v1_in),_v2(v2_in),_f(f_in) {}
     void operator()() const {
-        T1* first10=(T1*)at_c<0>(_v1.row_begin(0));
-        T1* first11=(T1*)at_c<1>(_v1.row_begin(0));
-        T1* first12=(T1*)at_c<2>(_v1.row_begin(0));
+        T1* first10=(T1*)boost::gil::at_c<0>(_v1.row_begin(0));
+        T1* first11=(T1*)boost::gil::at_c<1>(_v1.row_begin(0));
+        T1* first12=(T1*)boost::gil::at_c<2>(_v1.row_begin(0));
         T2* first2=(T1*)_v2.row_begin(0);
         T1* last2=first2+_v1.size()*3;
         while(first2!=last2) {
@@ -432,11 +435,14 @@ template <typename View1, typename View2, typename F>
 void test_transform(std::size_t trials) {
     image<typename View1::value_type, is_planar<View1>::value> im1(width,height);
     image<typename View2::value_type, is_planar<View2>::value> im2(width,height);
-    std::cout << "GIL: "    <<measure_time(transform_gil_t<View1,View2,F>(view(im1),view(im2),F()),trials) << std::endl;
-    std::cout << "Non-GIL: "<<measure_time(transform_nongil_t<View1,View2,F>(view(im1),view(im2),F()),trials) << std::endl;
+    //std::cout << "GIL: "    <<measure_time(transform_gil_t<View1,View2,F>(view(im1),view(im2),F()),trials) << std::endl;
+    //std::cout << "Non-GIL: "<<measure_time(transform_nongil_t<View1,View2,F>(view(im1),view(im2),F()),trials) << std::endl;
 }
 
-int main() {
+BOOST_AUTO_TEST_SUITE(GIL_Tests)
+
+BOOST_AUTO_TEST_CASE(performance_test)
+{
 #ifdef NDEBUG
     std::size_t num_trials=1000;
 #else
@@ -462,11 +468,11 @@ int main() {
 
     // for_each()
     std::cout<<"test for_each_pixel() on rgb8_image_t"<<std::endl;
-    test_for_each<rgb8_view_t,rgb_fr_t<bits8> >(num_trials);
+    test_for_each<rgb8_view_t,rgb_fr_t<uint8_t> >(num_trials);
     std::cout<<std::endl;
 
     std::cout<<"test for_each_pixel() on rgb8_planar_image_t"<<std::endl;
-    test_for_each<rgb8_planar_view_t,rgb_fr_t<bits8> >(num_trials);
+    test_for_each<rgb8_planar_view_t,rgb_fr_t<uint8_t> >(num_trials);
     std::cout<<std::endl;
 
     // copy()
@@ -492,20 +498,20 @@ int main() {
 
     // transform()
     std::cout<<"test transform_pixels() between rgb8_image_t and rgb8_image_t"<<std::endl;
-    test_transform<rgb8_view_t,rgb8_view_t,bgr_to_rgb_t<bits8,pixel<bits8,rgb_layout_t> > >(num_trials);
+    test_transform<rgb8_view_t,rgb8_view_t,bgr_to_rgb_t<uint8_t,pixel<uint8_t,rgb_layout_t> > >(num_trials);
     std::cout<<std::endl;
 
     std::cout<<"test transform_pixels() between rgb8_planar_image_t and rgb8_planar_image_t"<<std::endl;
-    test_transform<rgb8_planar_view_t,rgb8_planar_view_t,bgr_to_rgb_t<bits8,planar_pixel_reference<bits8,rgb_t> > >(num_trials);
+    test_transform<rgb8_planar_view_t,rgb8_planar_view_t,bgr_to_rgb_t<uint8_t,planar_pixel_reference<uint8_t,rgb_t> > >(num_trials);
     std::cout<<std::endl;
 
     std::cout<<"test transform_pixels() between rgb8_image_t and rgb8_planar_image_t"<<std::endl;
-    test_transform<rgb8_view_t,rgb8_planar_view_t,bgr_to_rgb_t<bits8,pixel<bits8,rgb_layout_t> > >(num_trials);
+    test_transform<rgb8_view_t,rgb8_planar_view_t,bgr_to_rgb_t<uint8_t,pixel<uint8_t,rgb_layout_t> > >(num_trials);
     std::cout<<std::endl;
 
     std::cout<<"test transform_pixels() between rgb8_planar_image_t and rgb8_image_t"<<std::endl;
-    test_transform<rgb8_planar_view_t,rgb8_view_t,bgr_to_rgb_t<bits8,planar_pixel_reference<bits8,rgb_t> > >(num_trials);
+    test_transform<rgb8_planar_view_t,rgb8_view_t,bgr_to_rgb_t<uint8_t,planar_pixel_reference<uint8_t,rgb_t> > >(num_trials);
     std::cout<<std::endl;
-
-    return 0;
 }
+
+BOOST_AUTO_TEST_SUITE_END()
