@@ -5,6 +5,7 @@ var searchData=
   ['adl_3c_20tuple_3c_3e_20_3e',['adl&lt; tuple&lt;&gt; &gt;',['../structboost_1_1hana_1_1detail_1_1operators_1_1adl.html',1,'boost::hana::detail::operators']]],
   ['adl_3c_3e',['adl&lt;&gt;',['../structboost_1_1hana_1_1detail_1_1operators_1_1adl.html',1,'boost::hana::detail::operators']]],
   ['any_5fof',['any_of',['../structboost_1_1hana_1_1detail_1_1any__of.html',1,'boost::hana::detail']]],
+  ['array',['array',['../structstd_1_1array.html',1,'std']]],
   ['array',['array',['../structboost_1_1hana_1_1detail_1_1array.html',1,'boost::hana::detail']]],
-  ['array',['array',['../structstd_1_1array.html',1,'std']]]
+  ['array_3c_20bool_2c_20sizeof_2e_2e_2e_28b_29_3e',['array&lt; bool, sizeof...(B)&gt;',['../structboost_1_1hana_1_1detail_1_1array.html',1,'boost::hana::detail']]]
 ];
